#include "gui.h"

MyMainFrame::MyMainFrame(const Setup* config_, const TGWindow *p,UInt_t w,UInt_t h): config(config_) {
   // use hierarchical cleaning
   fMain = new TGMainFrame(p,w,h);
   fMain->SetCleanup(kDeepCleanup);

   fMain->Connect("CloseWindow()", "MyMainFrame", this, "CloseWindow()");
   // create layout
   Init();
   canvas = fEcanvas->GetCanvas(); 
   canvas->Update();
   // popup window
   fMain->SetWindowName("Back projection reconstruction");
   fMain->MapSubwindows();
   fMain->Resize(fMain->GetDefaultSize());
   fMain->Resize(w, h);
   fMain->MapWindow();
}

MyMainFrame::~MyMainFrame() {
    // // Clean up used widgets: frames, buttons, layout hints
    // fMain->Cleanup();
    // menu
    delete fMenuBar;
    for (int i = 0; i < 3; i++)
    {
        delete fMenuEntries[i];
    }
    delete fToolBar;
    // canvas
    delete fF5;
    delete fL4;
    delete fEcanvas;
    delete histo;
    for (int i = 0; i < Nl; i++)
    {
        delete latitudes[i];
    }
    for (int i = 0; i < NL; i++)
    {
        delete longitudes[i];
    }
    delete sourceMarker;
    delete countsText;
    delete fMain;
}

void MyMainFrame::CloseWindow() {
   // Got close message for this MainFrame. Terminates the application.
   gApplication->Terminate(0);
}

int MyMainFrame::Init(){
    // create a menu bar
    fMenuBar = new TGMenuBar(fMain, 1, 1, kHorizontalFrame);
    // File menus
    fMenuEntries[0] = new TGPopupMenu(gClient->GetRoot());
    fMenuEntries[0]->AddEntry("&Open", M_CONFIG_OPEN);
    fMenuEntries[0]->AddSeparator();
    fMenuEntries[0]->AddEntry("&Save", M_IMAGE_SAVE);
    fMenuEntries[0]->AddEntry("S&ave as", M_IMAGE_SAVEAS);
    fMenuEntries[0]->AddSeparator();
    fMenuEntries[0]->AddEntry("&Exit", M_EXIT);
    fMenuBar->AddPopup("File", fMenuEntries[0], new TGLayoutHints(kLHintsTop | kLHintsLeft, 5, 5, 2, 2));
    fMenuEntries[0]->Connect("Activated(Int_t)", /* signal */
                      "MyMainFrame", this, /* receiver calls, object*/
                      "Response2Menu(Int_t)");

    fMenuEntries[1] = new TGPopupMenu(gClient->GetRoot());
    fMenuEntries[1]->AddEntry("Start", M_RUN_START);
    fMenuEntries[1]->AddEntry("Stop", M_RUN_STOP);
    fMenuEntries[1]->AddEntry("Clear", M_RUN_CLEAR);
    fMenuBar->AddPopup("Run", fMenuEntries[1], new TGLayoutHints(kLHintsTop | kLHintsLeft, 5, 5, 2, 2));
    fMenuEntries[1]->Connect("Activated(Int_t)", /* signal */
                      "MyMainFrame", this, /* receiver calls, object*/
                      "Response2Menu(Int_t)");

    fMenuEntries[2] = new TGPopupMenu(gClient->GetRoot());
    fMenuEntries[2]->AddEntry("About", M_ABOUT);
    fMenuEntries[2]->Connect("Activated(Int_t)", /* signal */
                      "MyMainFrame", this, /* receiver calls, object*/
                      "Response2Menu(Int_t)");
    fMenuBar->AddPopup("Help", fMenuEntries[2], new TGLayoutHints(kLHintsTop | kLHintsRight));

    fMain->AddFrame(fMenuBar, new TGLayoutHints(kLHintsExpandX, 5, 5, 2, 2) );
    TGHorizontal3DLine *lhMenu = new TGHorizontal3DLine(fMain);
    fMain->AddFrame(lhMenu, new TGLayoutHints(kLHintsTop|kLHintsExpandX));

    // toolbar
    fToolBar = new TGToolBar(fMain,520,80);
    for (int i = 0; i < 3; i++)
    {
        tooBarData[i].fPixmap = xmpicons[i].c_str();
        tooBarData[i].fTipText = tiptexts[i].c_str();
        tooBarData[i].fStayDown = kFALSE;
        tooBarData[i].fId = i+1;
        tooBarData[i].fButton = NULL;
        fToolBar->AddButton(fMain, &tooBarData[i], 5);
    }
    fToolBar->Connect("Clicked(Int_t)", /* signal */
                      "MyMainFrame", this, /* receiver calls, object*/
                      "Response2ToolBar(Int_t");
    fMain->AddFrame(fToolBar, new TGLayoutHints(kLHintsTop| kLHintsLeft| kLHintsExpandX, 10, 5, 2, 2));
    // adding a horizontal line as a separator
    TGHorizontal3DLine *lhToolBar = new TGHorizontal3DLine(fMain);
    fMain->AddFrame(lhToolBar, new TGLayoutHints(kLHintsTop|kLHintsExpandX));

    // embedded canvas
    fF5 = new TGCompositeFrame(fMain, 60, 60, kHorizontalFrame);
    fL4 = new TGLayoutHints(kLHintsTop | kLHintsLeft 
                            | kLHintsExpandX | kLHintsExpandY, 5, 5, 5, 5);
    fEcanvas = new TRootEmbeddedCanvas("ec1", fF5, 100, 100);
    fF5->AddFrame(fEcanvas, fL4);
    fMain->AddFrame(fF5, fL4);

    initHist();
    drawGridlines();
    drawSource();
    std::string strtmp = "Total counts: " + std::to_string(counts);
    countsText = new TText(0.7, 0.92, strtmp.c_str());
    countsText->SetTextSizePixels(2);
    countsText->SetNDC(kTRUE);
    countsText->Draw("SAME");
    // end embeded canvas
    return 0;
}

void MyMainFrame::SaveImage()
{
    if(!fout.fFilename)
    {
        SaveImageAs();
    }
    else
    {
        canvas->SaveAs(fout.fFilename);
        printf("Image saved to: %s\n", fout.fFilename);
    }
}

void MyMainFrame::histo2txt()
{
    std::ofstream outf;
    outf.open(fout.fFilename);
    if (!outf.good())
    {
        printf("cannot write to file: %s\n", fout.fFilename);
        return;
    }
    outf << "     Phi   Theta      Content\n";
    for (int i = 1; i <= histo->GetNbinsX(); i++)
    {
        for (int j = 1; j <= histo->GetNbinsY(); j++)
        {
            outf << std::fixed    << std::setprecision(2)
                 << std::setw(8)  << ((TAxis*)histo->GetXaxis())->GetBinCenter(i) 
                 << std::setw(8)  << ((TAxis*)histo->GetYaxis())->GetBinCenter(j)
                 << std::fixed    << std::setprecision(8)
                 << std::setw(13) << histo->GetBinContent(i,j) << '\n';
        }
    }
    printf("Image saved to: %s\n", fout.fFilename);
}
void MyMainFrame::SaveImageAs()
{
    static TString outdir(".");
    const char* WriteOutTypes[]={ "PNG",    "*.png", // id = 0
                                    "ROOT files",    "*.root", // id= 2
                                    "ROOT macros",   "*.C", // id =4
                                    "Text files",    "*.txt", // id =8
                                    0,               0 };
    fout.fFileTypes = WriteOutTypes;
    fout.SetIniDir(outdir);
    // printf("fIniDir = %s\n", fout.fIniDir);
    new TGFileDialog(gClient->GetRoot(), fMain, kFDSave, &fout);
    if (fout.fFilename)
    {
        outdir = fout.fIniDir;
        if (fout.fFileTypeIdx == 6)
        {
            histo2txt();
        }
        else
        {
            canvas->SaveAs(fout.fFilename);
            printf("Image saved to: %s\n", fout.fFilename);
        }        
    }
}
void MyMainFrame::Response2Menu(Int_t menu_id)
{
    switch (menu_id)
   {
    case M_CONFIG_OPEN:
    //   LoadConfig();
        printf("Load config! \n");
        break;
    case M_IMAGE_SAVE:
        SaveImage();
        break;
    case M_IMAGE_SAVEAS:
    //   SaveConfigAs();
        // printf("Save image as \n");
        SaveImageAs();
        break;
    case M_EXIT:
        CloseWindow();
        break;
    case M_RUN_START:
        Start();
        break;
    case M_RUN_STOP:
        Stop();
        break;
    case M_RUN_CLEAR:
        Clear();
        break;
    case M_ABOUT:
        printf("Show help. \n");
        break;
    default:
        break;
   }
}

void MyMainFrame::Response2ToolBar(Int_t button_id)
{
    switch (button_id)
    {
    case 1:
        Start();
        break;
    case 2:
        Clear();
        break;
    case 3:
        Stop();
        break;
    default:
        break;
   }
}
void MyMainFrame::initHist() {
    histo = new TH2D("ROI", " ; Azimuth (degree); Elevation (degree)", 
                            config->phiBins, -180, 180, 
                            config->thetaBins, -90, 90);
    // init image
    for (int i = 0; i < config->phiBins; i++)
    {
        for (int j = 0; j < config->thetaBins; j++)
        {
            histo->SetBinContent(i+1, j+1, 0);
        }
    }
    gStyle->SetOptStat(0);
    histo->GetZaxis()->SetLabelSize(0.02);
    histo->Draw("z aitoff");
}

void MyMainFrame::drawGridlines() {
    float conv=M_PI/180;
    float la, lo, x, y, z;
    for (int j=0;j<Nl;++j) {
        // latitudes[j]=new TGraph();
        latitudes.push_back(new TGraph());
        la = -90+180/(Nl-1)*j;
        for (int i=0;i<M+1;++i) {
            lo = -180+360/M*i;
            z  = sqrt(1+cos(la*conv)*cos(lo*conv/2));
            x  = 180*cos(la*conv)*sin(lo*conv/2)/z;
            y  = 90*sin(la*conv)/z;
            latitudes[j]->SetPoint(i,x,y);
        }
    } 
    
    for (int j=0;j<NL;++j) {
        // longitudes[j]=new TGraph();
        longitudes.push_back(new TGraph());
        lo = -180+360/(NL-1)*j;
        for (int i=0;i<M+1;++i) {
            la = -90+180/M*i;
            z  = sqrt(1+cos(la*conv)*cos(lo*conv/2));
            x  = 180*cos(la*conv)*sin(lo*conv/2)/z;
            y  = 90*sin(la*conv)/z;
            longitudes[j]->SetPoint(i,x,y);
        }
    }
    // Draw the grid	
    TPad *pad2 = new TPad("pad2","",0,0,1,1);
    pad2->SetFillStyle(4000);
    pad2->SetFillColor(0);
    pad2->SetBorderSize(0);
    pad2->SetFrameBorderMode(0);
    pad2->SetFrameLineColor(0); 
    pad2->SetFrameBorderMode(0);
    pad2->Draw();
    pad2->cd();
    Double_t ymin = -89.5;
    Double_t ymax = 89.5;
    Double_t dy = (ymax-ymin)/0.8; //10 per cent margins top and bottom
    Double_t xmin = -180;
    Double_t xmax = 180;
    Double_t dx = (xmax-xmin)/0.8; //10 per cent margins left and right

    pad2->Range(xmin-0.1*dx,ymin-0.1*dy,xmax+0.1*dx,ymax+0.1*dy);

    for (int j=0;j<Nl;++j) latitudes[j]->Draw("l");
    for (int j=0;j<NL;++j) longitudes[j]->Draw("l");
}

void MyMainFrame::drawSource() {
    // draw source marker
    float conv=M_PI/180;
    double sigma = 1;
    double Ax, Ay;
    aitoff2xy(config->truePhi / conv, config->trueTheta / conv, Ax, Ay);
    sourceMarker = new TEllipse(Ax, Ay, sigma, sigma);
    sourceMarker->SetFillColor(0);
    sourceMarker->SetFillStyle(0);
    sourceMarker->SetLineWidth(2);
    sourceMarker->SetLineColor(kRed);
    sourceMarker->Draw("SAME");
}

void MyMainFrame::Start()
{
//    printf("\"Start updating\" Button Pressed!\n");
   stop = false;
}

void MyMainFrame::Stop()
{
//    printf("\"Stop updating\" Button Pressed!\n");
   stop = true;
}

void MyMainFrame::Clear()
{
//    printf("\"Clear image\" Button Pressed!\n");
   counts = 0;
   histo->Reset();
   redraw();
}

void MyMainFrame::updateImage(std::vector<Cone>::const_iterator first, 
                              std::vector<Cone>::const_iterator last,
                              const bool& normalized)
{
    if (first == last)
        return;
    // update image
    if (normalized)
        addConesNormalized(config, histo, counts, first, last);
    else
        addCones(config, histo, counts, first, last);
    // redraw
    redraw();
}

void MyMainFrame::redraw() {
    std::string strtmp = "Total counts: " + std::to_string(counts);
    countsText->SetText(0.7, 0.92, strtmp.c_str());
    canvas->Modified();
    canvas->Update();
}

int MyMainFrame::aitoff2xy(const double& l, const double& b, double &Al, double &Ab)
{
   Double_t x, y;

   Double_t alpha2 = (l/2)*TMath::DegToRad();
   Double_t delta  = b*TMath::DegToRad();
   Double_t r2     = TMath::Sqrt(2.);
   Double_t f      = 2*r2/TMath::Pi();
   Double_t cdec   = TMath::Cos(delta);
   Double_t denom  = TMath::Sqrt(1. + cdec*TMath::Cos(alpha2));
   x      = cdec*TMath::Sin(alpha2)*2.*r2/denom;
   y      = TMath::Sin(delta)*r2/denom;
   x     *= TMath::RadToDeg()/f;
   y     *= TMath::RadToDeg()/f;
   //  x *= -1.; // for a skymap swap left<->right
   Al = x;
   Ab = y;

   return 0;
}