#include <vector>
#include <iostream>
#include <cstdlib>
#include <ctime>

#include "TSystem.h"
#include "TApplication.h"

#include "setup.h"
#include "worker.h"
#include "gui.h"

int main(int argc, char** argv) {
    TApplication *myapp = new TApplication("myApp", 0, 0);
    // init
    Setup* config = new Setup(Vector3D(3.6, -10, 0));

    // simulate some cones on worker thread
    // or read data file to get the cones
    SPSC* coneQueue = new SPSC(config->capacity);
    Worker* worker = new Worker(config, coneQueue);

    // draw a projection image on master thread
    // pop up window
    MyMainFrame* myMainFrame = new MyMainFrame(config,gClient->GetRoot(),1000, 1000);
    std::vector<Cone> cones(config->chuckSize);
    bool unfinished = true;
    while (true)
    {
        if (!myMainFrame->stop)
        {
            int i(0);
            while (i < config->chuckSize)
            {
                unfinished = coneQueue->wait_dequeue_timed(cones[i], 
                                std::chrono::milliseconds(10));
                if (!unfinished) break;
                i++;
            }
            // update image
            myMainFrame->updateImage(cones.cbegin(), cones.cbegin() + i, true);
            if (!unfinished)
            {
                // stop updating after this final run
                myMainFrame->stop = true;
                std::cout << "Processing finished." << std::endl;
            }
        }
        // gSystem->Sleep(100);
        gSystem->ProcessEvents();
    }

    myapp->Run();

    delete worker;
    delete config;
    delete coneQueue;

    return 0;
}